/**
 * Edmonds-Karp 
 * 
 * Complexity (time): O(V*E^2)
 * Complexity (space): O(V^2)
 */

int N;
int par[MAX];
int graph[MAX][MAX], rg[MAX][MAX];

bool cont[MAX];

// Finds if there's a path between s and t using non-full
// residual edges
bool bfs(int s, int t) {
  queue<int> Q;
  Q.push(s);
  cont[s] = true;

  while (!Q.empty()) {
    int u = Q.front(); Q.pop();

    if (u == t)
      return true;

    for (int i = 0; i < N; ++i)
      if (!cont[i] && rg[u][i]) {
        cont[i] = true;
        par[i] = u;
        Q.push(i);
      }
  }

  return false;
}


// Returns maximum flow between s (source) and t (sink)
int edmonds_karp(int s, int t) {
  int ans = 0;
  par[s] = -1;

  mset(cont, 0);
  memcpy(rg, graph, sizeof(graph));

  // Repeat while there's a valid path between s and t
  while (bfs(s, t)) {
    int flow = inf;

    // Get the minimum capacity among all edges of the chosen path
    for (int i = t; par[i] != -1; i = par[i])
      flow = min(flow, rg[par[i]][i]);

    // Update residual graph
    for (int i = t; par[i] != -1; i = par[i]) {
      rg[par[i]][i] -= flow;
      rg[i][par[i]] += flow;
    }

    ans += flow;
    mset(cont, 0);
  }

  return ans;
}
